#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <qdos.h>
#include "q40snd.h"
#include "version.h"

char _prog_name[] = "playsnd " VERSION;
long _pipesize = MAXIO;

#define strcasecmp stricmp

int (*_conread)() = NULL;
int (*_conwrite)() = NULL;
char _conname[] = "nul";

static int factor = 800;

int makenoise (void *buf, int nsz)
{
    int j, n, res = 0;

    do
    {
	n = sndplay (buf, nsz);
	if (n < 0)
	{
	    res = n;
	    break;
	}
	else if (n)
	{
	    int nl = nsz - n;

	    j = sndbusy();
	    j = j/factor - 2;
	    if (j > 0)
		mt_susjb (-1, j, 0);
	    (char *) buf = (char *) buf + nl;
	    nsz = n;
	}
    }
    while (n);
    return res;
}

int process(int fd, void *buf, int maxb)
{
    int rn = 0;
    int n;
    while (rn == 0 && (n = read (fd, buf, maxb)) > 0)
    {
        n &= ~1;
        rn = makenoise (buf, n);
    }
    if (n < 0)
        rn = n;
    return rn;
}

int main(int ac, char **av)
{
    char *p,cmd[256];
    int nsox = 0;
    int fd;
    struct stat st;
    char *lu = NULL, *lp = NULL;
    const char *tmpfn = "ram1_playsnd_log";
    char *ifnam = NULL;
    int maxb = (100*1024);
        
    if(ac < 2) exit(ERR_BP);
    if((p = getenv("SND_FACTOR")) != NULL)
    {
        int f = atoi(p);
        if(f > 400 && f < 4000)
        {
            factor = f;
        }
    }
   
    freopen(tmpfn, "a", stderr);
    ifnam = *(av+ac-1);

    if(strcmp(ifnam,"-") != 0)
    {
        lu = strrchr(ifnam,'.');

        if(lu == NULL)
        {
            lp = lu = strrchr(ifnam, '_');
        }

        if(lu)
        {
            nsox = (ac == 2 && strcasecmp(lu+1,"ub") == 0);
        }
        if((fd = open(ifnam, O_RDONLY)) >= 0)
        {
            if(0 == fstat(fd, &st))
            {
                maxb = MIN (st.st_size, maxb);
                maxb &= ~1;
            }
            else
                maxb = 0;
            if(nsox == 0)
                close(fd);
        }
        else
        {
            fd = fileno(stdin);
        }
    }
    
    if(fd >= 0)
    {
        if(maxb)
        {
            char *buf;

            if ((buf = malloc (maxb)) != NULL)
            {
                if(nsox)
                {
                    process(fd, buf, maxb);
                }
                else
                {
                    int pf[2];
                    int nc = 0;
                    short opt_t = 0;
                    char *sav[64];
                    
                    *(sav + nc) = "sox"; nc++;
                    
                    ac--;
                    while(++av, --ac)
                    {
                        if(**av == '-' && *((*av)+1) == 't')
                        {
                            opt_t = 1;
                        }
                        *(sav + nc) = *av; nc++;
                    }
                    if(opt_t == 0 && lp)
                    {
                        char *tbuf = malloc(32);
                        stpcpy(tbuf,"-t.");
                        strcpy(tbuf+3, lp+1);
                        *(sav + nc) = tbuf; nc++;
                    }
                    *(sav + nc) = *(av+ac); nc++;                
                    *(sav + nc) = "-c2"; nc++;
                    *(sav + nc) = "-tub"; nc++;                                
                    *(sav + nc) = "-r20000"; nc++;
                    *(sav + nc) = "-"; nc++;                                
                    *(sav + nc) = NULL;
                    
                            
                    if(0 == pipe(pf))
                    {
                        int chans[4];
                        chans[0] = 3;
                        chans[1] = 0;
                        chans[2] = 2;
                        chans[3] = pf[1];
                        if(forkvp("sox",chans,sav) > 0)
                        {
                            process(pf[0], buf, maxb);
                            close(pf[1]);
                            close(pf[0]);
                            while( wait( NULL ) != -1)
                                ; /* foo */
                            fclose(stderr);
                            stat(tmpfn, &st);
                            if(st.st_size == 0)
                            {
                                unlink(tmpfn);
                            }
                        }
                    }
                }
                free(buf);
            }
        }
    }
    return _oserr;
}
