;;; ------------------------------------------------------------------
;;; 
;;;  q40 sound functions
;;; 
;;; 
;;; ------------------------------------------------------------------
		
	SSS_ADD1	equ	$04 ; add single sample
	SSS_SETM	equ	$08 ; set up queue
	SSS_ADDM	equ	$0c ; add samples to queue
	SSS_KILL	equ	$10 ; kill sound system
	SSS_SAMPLE	equ	$14 ; get queue size
	
	SSS_ID	equ	$53535353   ; 'SSSS'

	.text

	.globl	_sndplay
	.globl	_sndbusy
	.globl	_sndkill
	.globl	_sndadd1
	

;;; For Harry Potter fans ...
hogwarts:	
dc.l	$4e7ab801		; magic instruction (movec vbr,a3)
	moveq	#0,d0		; assume it works
	move.l	$70(a3),a3	; deeper magic
	move.l	-(a3),a2	; start of functions
	cmp.l	#SSS_ID,-(a3)	; do we have sound
	beq	hedwig		; yes
	moveq	#-19,d0		; not implemented, alas
hedwig:	
	tst.l	d0
	rts			

;;; ------------------------------------------------------------------
;;;
;;; res = playsnd(void *sample, int length)
;;;
;;; res:
;;;	 -ve	error
;;;	0	All data played
;;;	+ve	samples remaining (in bytes!!!)
;;; 
;;; ------------------------------------------------------------------
_sndplay:	
	movem.l	 d2/a2-a3,-(sp)
	move.l	4+12(sp),a0	; source
	move.l	4+16(sp),d2	; length
	ble	noplay		; no way

	trap	#0		; supervisor, alas

	bsr	hogwarts	; do SSS magic
	bne	userland	; can't so be a muggle again

	move.l	d2,d0		; user sample size (bytes)
	jsr	SSS_SETM(a2)	; request queue (returns start A1, end A2)
	bra	eloop		; go

tloop:	
	move.b	(a0)+,(a1)+	; left byte
	move.b	(a0)+,(a1)+	; right byte
	subq.l	#2, d0		; off queue

eloop:	
	tst.l	d0		; done ?
	ble	goplay		; all our samples (should not be -ve!)
	cmp.l	a2,a1		; queue full
	blt	tloop

;;;	Here we are all set to play
	
goplay:	
	move.l	d0,d2		; save count
	bsr	hogwarts	; do SSS magic (again)
	bne	userland	; can't happen!
	jsr	SSS_ADDM(a2)	; assume we can play without further
	move.l	d2,d0		; restore count
userland:
	andi.w	#$d8ff,sr	; user mode again
done:
	movem.l	(sp)+,d2/a2-a3
	rts
noplay:
	moveq	#-15,d0		; bad parameter
	bra	done
	
;;; ------------------------------------------------------------------
;;; res = sndbusy(void);
;;; 
;;;  res = samples in queue (n.b. it's stereo)
;;;  hence
;;;   time to play ~= res/20000 secs
;;;   ticks to play ~= res/400
;;; ------------------------------------------------------------------
	 
_sndbusy:
	movem.l	 d2/a2-a3,-(sp)
	trap	#0		; supervisor, alas
	bsr	hogwarts
	bne	userland
	jsr	SSS_SAMPLE(a2)	; request queue
	bra	userland	; done (d0 = count)
	
;;; ------------------------------------------------------------------
;;; void sndkill(void);
;;;
;;; Abort sound system, throwing away queue
;;; ------------------------------------------------------------------

_sndkill:
	movem.l	 d2/a2-a3,-(sp)
	trap	#0		; supervisor, alas
	bsr	hogwarts
	bne	userland
	jsr	SSS_KILL(a2)	; request queue
	bra	userland	; done 

;;; ------------------------------------------------------------------
;;; void sndadd1(void *);
;;; ------------------------------------------------------------------
		
_sndadd1:	
	movem.l	 d2/a2-a3,-(sp)
	move.l	4(sp),a1
	trap	#0		; supervisor, alas
	bsr	hogwarts
	bne	userland
	move.b	(a1),d1
	move.b	1(a1),d2
	jsr	SSS_ADD1(a2)	; request queue
	bra	userland	; done 
